package com.ruoyi.branch.service;

import java.util.List;
import com.ruoyi.branch.domain.TBranchMember;

/**
 * 支部成员管理Service接口
 * 
 * @author ruoyi
 * @date 2023-06-15
 */
public interface ITBranchMemberService 
{
    /**
     * 查询支部成员管理
     * 
     * @param memberId 支部成员管理主键
     * @return 支部成员管理
     */
    public TBranchMember selectTBranchMemberByMemberId(Long memberId);

    /**
     * 查询支部成员政治生日列表
     *
     * @param tBranchMember 支部成员管理
     * @return 支部成员管理集合
     */
    public List<TBranchMember> selectPoliticalBirthdayList(TBranchMember tBranchMember);

    /**
     * 查询支部成员管理列表
     * 
     * @param tBranchMember 支部成员管理
     * @return 支部成员管理集合
     */
    public List<TBranchMember> selectTBranchMemberList(TBranchMember tBranchMember);

    /**
     * 新增支部成员管理
     * 
     * @param tBranchMember 支部成员管理
     * @return 结果
     */
    public int insertTBranchMember(TBranchMember tBranchMember);

    /**
     * 修改支部成员管理
     * 
     * @param tBranchMember 支部成员管理
     * @return 结果
     */
    public int updateTBranchMember(TBranchMember tBranchMember);

    /**
     * 批量删除支部成员管理
     * 
     * @param memberIds 需要删除的支部成员管理主键集合
     * @return 结果
     */
    public int deleteTBranchMemberByMemberIds(Long[] memberIds);

    /**
     * 删除支部成员管理信息
     * 
     * @param memberId 支部成员管理主键
     * @return 结果
     */
    public int deleteTBranchMemberByMemberId(Long memberId);
}
